@@ -1,5 +1,5 @@
 /*
- *  Copyright 2013  Andreas Cord-Landwehr <cordlandwehr@kde.org>
+ *  Copyright 2013-2016  Andreas Cord-Landwehr <cordlandwehr@kde.org>
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU Lesser General Public
@@ -20,11 +20,12 @@
 
 #include "storage.h"
 #include "learner.h"
-
 #include "liblearner_debug.h"
-#include <QDir>
+
 #include <KLocalizedString>
 
+#include <QDateTime>
+#include <QDir>
 #include <QSqlError>
 #include <QSqlDatabase>
 #include <QSqlQuery>
@@ -34,11 +35,21 @@ using namespace LearnerProfile;
 
 Storage::Storage(QObject* parent)
     : QObject(parent)
+    , m_databasePath(QStandardPaths::writableLocation(
+        QStandardPaths::DataLocation) + QLatin1Char('/') + "learnerdata.db")
     , m_errorMessage(QString())
 {
 
 }
 
+Storage::Storage(const QString databasePath, QObject* parent)
+    : QObject(parent)
+    , m_databasePath(databasePath)
+    , m_errorMessage(QString())
+{
+    qCDebug(LIBLEARNER_LOG) << "Initialize with custom DB path:" << m_databasePath;
+}
+
 QString Storage::errorMessage() const
 {
     return m_errorMessage;
@@ -127,7 +138,7 @@ bool Storage::storeProfile(Learner *learner)
     cleanupRelations.prepare("DELETE FROM learner_goals WHERE ");
     //TODO change creation of relations to same way as remove-relations: explicit connections
 
-    return false;
+    return true;
 }
 
 bool Storage::removeProfile(Learner *learner)
@@ -165,7 +176,6 @@ bool Storage::removeProfile(Learner *learner)
 
 bool Storage::removeRelation(Learner *learner, LearningGoal *goal)
 {
-qCDebug(LIBLEARNER_LOG) << "remove relation";
     QSqlDatabase db = database();
     QSqlQuery removeGoalRelationQuery(db);
     removeGoalRelationQuery.prepare(
@@ -216,8 +226,8 @@ QList< Learner* > Storage::loadProfiles(QList<LearningGoal*> goals)
         return QList<Learner*>();
     }
     while (goalRelationQuery.next()) {
-        Learner *learner = 0;
-        LearningGoal *goal = 0;
+        Learner *learner = nullptr;
+        LearningGoal *goal = nullptr;
 
         foreach (Learner *cmpProfile, profiles) {
             if (cmpProfile->identifier() == goalRelationQuery.value(2).toInt()) {
@@ -225,6 +235,10 @@ QList< Learner* > Storage::loadProfiles(QList<LearningGoal*> goals)
                 break;
             }
         }
+        if (!learner) {
+            qCCritical(LIBLEARNER_LOG) << "Could not retrieve learner from database.";
+            return QList<Learner*>();
+        }
         foreach (LearningGoal *cmpGoal, goals) {
             if (cmpGoal->category() == goalRelationQuery.value(0).toInt()
                 && cmpGoal->identifier() == goalRelationQuery.value(1).toString())
@@ -237,8 +251,7 @@ QList< Learner* > Storage::loadProfiles(QList<LearningGoal*> goals)
         if (learner->goals().contains(goal)) {
             continue;
         }
-
-        if (goal != 0 && learner != 0) {
+        if (goal) {
             learner->addGoal(goal);
         }
     }
@@ -271,7 +284,6 @@ bool Storage::storeGoal(LearningGoal *goal)
         insertGoalQuery.bindValue(1, goal->identifier());
         insertGoalQuery.bindValue(2, goal->name());
         insertGoalQuery.exec();
-
         if (insertGoalQuery.lastError().isValid()) {
             raiseError(insertGoalQuery.lastError());
             db.rollback();
@@ -320,23 +332,215 @@ QList< LearningGoal* > Storage::loadGoals()
     return goals;
 }
 
+bool Storage::storeProgressLog(Learner *learner, LearningGoal *goal,
+                            const QString &container, const QString &item, int payload,
+                            const QDateTime &time)
+{
+    QSqlDatabase db = database();
+    QSqlQuery insertQuery(db);
+    insertQuery.prepare("INSERT INTO learner_progress_log "
+        "(goal_category, goal_identifier, profile_id, item_container, item, payload, date) "
+        "VALUES (:gcategory, :gidentifier, :pid, :container, :item, :payload, :date)");
+    insertQuery.bindValue(":gcategory", static_cast<int>(goal->category()));
+    insertQuery.bindValue(":gidentifier", goal->identifier());
+    insertQuery.bindValue(":pid", learner->identifier());
+    insertQuery.bindValue(":container", container);
+    insertQuery.bindValue(":item", item);
+    insertQuery.bindValue(":payload", payload);
+    insertQuery.bindValue(":date", time.toString(Qt::ISODate));
+    insertQuery.exec();
+
+    if (insertQuery.lastError().isValid()) {
+        raiseError(insertQuery.lastError());
+        qCCritical(LIBLEARNER_LOG) << "DB Error:" << m_errorMessage;
+        db.rollback();
+        return false;
+    }
+    return true;
+}
+
+QList<QPair<QDateTime,int>> Storage::readProgressLog(Learner *learner, LearningGoal *goal,
+                            const QString &container, const QString &item)
+{
+    QSqlDatabase db = database();
+    QSqlQuery logQuery(db);
+    logQuery.prepare("SELECT date, payload FROM learner_progress_log "
+        "WHERE goal_category = :goalcategory "
+        "AND goal_identifier = :goalid "
+        "AND profile_id = :profileid "
+        "AND item_container = :container "
+        "AND item = :item");
+    logQuery.bindValue(":goalcategory", static_cast<int>(goal->category()));
+    logQuery.bindValue(":goalid", goal->identifier());
+    logQuery.bindValue(":profileid", learner->identifier());
+    logQuery.bindValue(":container", container);
+    logQuery.bindValue(":item", item);
+    logQuery.exec();
+    if (logQuery.lastError().isValid()) {
+        qCritical() << logQuery.lastError().text();
+        raiseError(logQuery.lastError());
+        return QList<QPair<QDateTime,int>>();
+    }
+
+    QList<QPair<QDateTime,int>> log;
+    while (logQuery.next()) {
+        const QDateTime date{logQuery.value(0).toDateTime()};
+        int payload{logQuery.value(1).toInt()};
+        log.append(qMakePair(date, payload));
+    }
+    return log;
+}
+
+bool Storage::storeProgressValue(Learner *learner, LearningGoal *goal,
+                            const QString &container, const QString &item, int payload)
+{
+    QSqlDatabase db = database();
+    QSqlQuery query(db);
+
+    // test if already payload stored
+    query.prepare("SELECT payload FROM learner_progress_value "
+        "WHERE goal_category = :gcategory "
+        "AND goal_identifier = :gidentifier "
+        "AND profile_id = :pid "
+        "AND item_container = :container "
+        "AND item = :item");
+    query.bindValue(":gcategory", static_cast<int>(goal->category()));
+    query.bindValue(":gidentifier", goal->identifier());
+    query.bindValue(":pid", learner->identifier());
+    query.bindValue(":container", container);
+    query.bindValue(":item", item);
+    query.exec();
+    if (query.lastError().isValid()) {
+        qCritical() << query.lastError().text();
+        raiseError(query.lastError());
+        return false;
+    }
+    // if query contains values, perform update query
+    if (query.next()) {
+        query.finish(); // release resources from previous query
+        query.prepare("UPDATE learner_progress_value "
+            "SET payload = :payload "
+            "WHERE goal_category = :gcategory "
+            "AND goal_identifier = :gidentifier "
+            "AND profile_id = :pid "
+            "AND item_container = :container "
+            "AND item = :item");
+        query.bindValue(":payload", static_cast<int>(payload));
+        query.bindValue(":gcategory", static_cast<int>(goal->category()));
+        query.bindValue(":gidentifier", goal->identifier());
+        query.bindValue(":pid", learner->identifier());
+        query.bindValue(":container", container);
+        query.bindValue(":item", item);
+        query.exec();
+
+        if (query.lastError().isValid()) {
+            qCritical() << query.lastError().text();
+            raiseError(query.lastError());
+            db.rollback();
+            return false;
+        }
+        return true;
+    }
+    // else insert new row
+    else {
+        query.finish(); // release resources from previous query
+        query.prepare("INSERT INTO learner_progress_value "
+            "(goal_category, goal_identifier, profile_id, item_container, item, payload) "
+            "VALUES (:gcategory, :gidentifier, :pid, :container, :item, :payload)");
+        query.bindValue(":gcategory", static_cast<int>(goal->category()));
+        query.bindValue(":gidentifier", goal->identifier());
+        query.bindValue(":pid", learner->identifier());
+        query.bindValue(":container", container);
+        query.bindValue(":item", item);
+        query.bindValue(":payload", static_cast<int>(payload));
+        query.exec();
+
+        if (query.lastError().isValid()) {
+            qCritical() << query.lastError().text();
+            raiseError(query.lastError());
+            db.rollback();
+            return false;
+        }
+        return true;
+    }
+    Q_UNREACHABLE();
+    return false;
+}
+
+QHash<QString,int> Storage::readProgressValues(Learner *learner, LearningGoal *goal,
+                            const QString &container)
+{
+    QSqlDatabase db = database();
+    QSqlQuery query(db);
+    query.prepare("SELECT item, payload FROM learner_progress_value "
+        "WHERE goal_category = :goalcategory "
+        "AND goal_identifier = :goalid "
+        "AND profile_id = :profileid "
+        "AND item_container = :container");
+    query.bindValue(":goalcategory", static_cast<int>(goal->category()));
+    query.bindValue(":goalid", goal->identifier());
+    query.bindValue(":profileid", learner->identifier());
+    query.bindValue(":container", container);
+    query.exec();
+    if (query.lastError().isValid()) {
+        qCritical() << query.lastError().text();
+        raiseError(query.lastError());
+        return QHash<QString,int>();
+    }
+
+    QHash<QString,int> values;
+    while (query.next()) {
+        const QString item{query.value(0).toString()};
+        const int payload{query.value(1).toInt()};
+        values.insert(item, payload);
+    }
+    return values;
+}
+
+int Storage::readProgressValue(Learner *learner, LearningGoal *goal,
+                            const QString &container, const QString &item)
+{
+    QSqlDatabase db = database();
+    QSqlQuery query(db);
+    query.prepare("SELECT payload FROM learner_progress_value "
+        "WHERE goal_category = :goalcategory "
+        "AND goal_identifier = :goalid "
+        "AND profile_id = :profileid "
+        "AND item_container = :container "
+        "AND item = :item");
+    query.bindValue(":goalcategory", static_cast<int>(goal->category()));
+    query.bindValue(":goalid", goal->identifier());
+    query.bindValue(":profileid", learner->identifier());
+    query.bindValue(":container", container);
+    query.bindValue(":item", item);
+    query.exec();
+    if (query.lastError().isValid()) {
+        qCritical() << query.lastError().text();
+        raiseError(query.lastError());
+        return -1;
+    }
+
+    if (query.next()) {
+        return query.value(0).toInt();
+    }
+    return -1;
+}
+
 QSqlDatabase Storage::database()
 {
     if (QSqlDatabase::contains(QSqlDatabase::defaultConnection)) {
         return QSqlDatabase::database(QSqlDatabase::defaultConnection);
     }
 
-    QString path = QStandardPaths::writableLocation(QStandardPaths::DataLocation) + QLatin1Char('/') + "learnerdata.db";
-
     // create data directory if it does not exist
     QDir dir = QDir(QStandardPaths::writableLocation(QStandardPaths::DataLocation));
     if (!dir.exists()) {
         dir.mkpath(QStandardPaths::writableLocation(QStandardPaths::DataLocation));
     }
-    qCDebug(LIBLEARNER_LOG) << "Database path: " << path;
+    qCDebug(LIBLEARNER_LOG) << "Database path: " << m_databasePath;
 
     QSqlDatabase db = QSqlDatabase::addDatabase("QSQLITE");
-    db.setDatabaseName(path);
+    db.setDatabaseName(m_databasePath);
     if (!db.open()) {
         qCritical() << "Could not open database: " << db.lastError().text();
         raiseError(db.lastError());
@@ -438,5 +642,38 @@ bool Storage::updateSchema()
         return false;
     }
 
+    // table for full progress data log
+    db.exec("CREATE TABLE IF NOT EXISTS learner_progress_log ("
+            "id INTEGER PRIMARY KEY AUTOINCREMENT, "
+            "goal_category INTEGER, "    // LearningGoal::Category
+            "goal_identifier TEXT, "     // LearningGoal::Identifier
+            "profile_id INTEGER, "       // Learner::Identifier
+            "item_container TEXT, "
+            "item TEXT, "
+            "payload INTEGER, "
+            "date TEXT"
+            ")");
+    if (db.lastError().isValid()) {
+        qCritical() << db.lastError().text();
+        raiseError(db.lastError());
+        return false;
+    }
+
+    // table for progress data quick access
+    db.exec("CREATE TABLE IF NOT EXISTS learner_progress_value ("
+            "id INTEGER PRIMARY KEY AUTOINCREMENT, "
+            "goal_category INTEGER, "    // LearningGoal::Category
+            "goal_identifier TEXT, "     // LearningGoal::Identifier
+            "profile_id INTEGER, "       // Learner::Identifier
+            "item_container TEXT, "
+            "item TEXT, "
+            "payload INTEGER"
+            ")");
+    if (db.lastError().isValid()) {
+        qCritical() << db.lastError().text();
+        raiseError(db.lastError());
+        return false;
+    }
+
     return true;
 }
