@@ -31,29 +31,20 @@
 #include "liblearnerprofile/src/profilemanager.h"
 #include "liblearnerprofile/src/learner.h"
 #include "libsound/src/outputdevicecontroller.h"
+#include "artikulate_debug.h"
 
 #include <KActionCollection>
 #include <KConfigDialog>
-#include <KDeclarative/KDeclarative>
 #include <KHelpMenu>
 #include <KLocalizedString>
-#include <KMessageBox>
+#include <KLocalizedContext>
 #include <KNS3/DownloadDialog>
 #include <KStandardAction>
 
 #include <QAction>
-#include <QApplication>
-#include <QCloseEvent>
-#include <QCoreApplication>
-#include "artikulate_debug.h"
-#include <QGraphicsObject>
 #include <QIcon>
-#include <QMenuBar>
 #include <QPointer>
 #include <QQmlContext>
-#include <QQmlProperty>
-#include <QQuickItem>
-#include <QQuickView>
 #include <QStandardPaths>
 
 using namespace LearnerProfile;
@@ -62,9 +53,11 @@ MainWindow::MainWindow()
     : m_actionCollection(new KActionCollection(this, "artikulate"))
     , m_helpMenu(new KHelpMenu)
     , m_resourceManager(new ResourceManager(this))
-    , m_trainingSession(new TrainingSession(this))
     , m_profileManager(new LearnerProfile::ProfileManager(this))
+    , m_trainingSession(new TrainingSession(this))
 {
+    rootContext()->setContextObject(new KLocalizedContext(this));
+
     // load saved sound settings
     OutputDeviceController::self().setVolume(Settings::audioOutputVolume());
 
@@ -75,10 +68,7 @@ MainWindow::MainWindow()
     }
     m_resourceManager->loadCourseResources();
     m_resourceManager->registerLearningGoals(m_profileManager);
-
-    KDeclarative::KDeclarative kdeclarative;
-    kdeclarative.setDeclarativeEngine(this);
-    kdeclarative.setupBindings(); //TODO use result for determining touch/desktop version
+    m_trainingSession->setProfileManager(m_profileManager);
 
     // create menu
     setupActions();
@@ -86,12 +76,12 @@ MainWindow::MainWindow()
     // set view
     rootContext()->setContextProperty("g_resourceManager", m_resourceManager);
     rootContext()->setContextProperty("g_trainingSession", m_trainingSession);
-    rootContext()->setContextProperty("profileManager", m_profileManager);
+    rootContext()->setContextProperty("g_profileManager", m_profileManager);
     rootContext()->setContextProperty("kcfg_UseContributorResources", Settings::useCourseRepository());
     rootContext()->setContextProperty("kcfg_ShowMenuBar", Settings::showMenuBar());
 
     // set starting screen
-    load(QUrl::fromLocalFile(QStandardPaths::locate(QStandardPaths::DataLocation, "qml/Main.qml")));
+    load(QUrl("qrc:/artikulate/qml/Main.qml"));
 
     // settings from kcfg values
 //     updateTrainingPhraseFont(); //FIXME deactivated while porting
